#include "bodypart.h"

Bodypart::Bodypart(btDynamicsWorld* ownerWorld, void* owner, const btVector3& dimensions, float weight, btTransform& offset, btTransform& transform)
{
	m_ownerWorld = ownerWorld;

	shape = new btBoxShape( dimensions );

	btVector3 localInertia(0,0,0);
	if (weight != 0.f) // weight of non zero = dynamic
		shape->calculateLocalInertia(weight,localInertia);

//	for (unsigned int i=0;i<=2;i++){
//		localInertia[1] *= -1.1f;
//		cout << "L-I "<<localInertia[1]<< "("<<1<<")"<<endl;
//	}
	

	btDefaultMotionState* myMotionState = new btDefaultMotionState(offset*transform);
	
	btRigidBody::btRigidBodyConstructionInfo rbInfo(weight,myMotionState,shape,localInertia);
	body = new btRigidBody(rbInfo);

	body->setUserPointer(owner);
	body->setDamping(0.05, 0.85);
	body->setDeactivationTime(0.001);
	body->setSleepingThresholds(1.6, 2.5);

	m_ownerWorld->addRigidBody(body);

}

Bodypart::Bodypart(btDynamicsWorld* ownerWorld, void* owner, const btVector3& dimensions, float weight, btTransform& offset, btTransform& transform, float xdest, float ydest, float zdest, float speed, float rx, float ry, float rz)
{
	m_ownerWorld = ownerWorld;

	shape = new btBoxShape( dimensions );

	btVector3 localInertia(0,0,0);
	if (weight != 0.f) // weight of non zero = dynamic
		shape->calculateLocalInertia(weight,localInertia);

//	for (unsigned int i=0;i<=2;i++){
//		localInertia[1] *= -1.1f;
//		cout << "L-I "<<localInertia[1]<< "("<<1<<")"<<endl;
//	}
	

	btDefaultMotionState* myMotionState = new btDefaultMotionState(offset*transform);
	
	btRigidBody::btRigidBodyConstructionInfo rbInfo(weight,myMotionState,shape,localInertia);
	body = new btRigidBody(rbInfo);

	body->setUserPointer(owner);
	body->setDamping(0.05, 0.85);
	body->setDeactivationTime(0.001);
	body->setSleepingThresholds(1.6, 2.5);

	btVector3 linVel(xdest,ydest,zdest);
	linVel.normalize();
	linVel*=speed;

//	body->getWorldTransform().setRotation(btQuaternion(0,0,0,1));
	body->getWorldTransform().setRotation(btQuaternion(rz,rx,ry,1));
	body->setLinearVelocity(linVel);
	body->setAngularVelocity(btVector3(rx,ry,rz));
//	body->setAngularVelocity(btVector3(0,0,0));
//	body->setCcdMotionThreshold(1.);
//	body->setCcdSweptSphereRadius(0.2f);

	m_ownerWorld->addRigidBody(body);

}



Bodypart::~Bodypart()
{
	// Remove all bodies and shapes
	m_ownerWorld->removeRigidBody(body);
	m_ownerWorld->getBroadphase()->getOverlappingPairCache()->cleanProxyFromPairs(body->getBroadphaseHandle(),m_ownerWorld->getDispatcher());
	delete body->getMotionState();
	delete body;
	delete shape;
}


